// snowglobe includes pieces modified from 
//      File > Examples > Adafruit NeoPixels > strandtest and 
//      File > Examples > Adafruit Circuit Playground > FidgetSpinner

// NEOPIXEL BEST PRACTICES for most reliable operation:
// - Add 1000 uF CAPACITOR between NeoPixel strip's + and - connections.
// - MINIMIZE WIRING LENGTH between microcontroller board and first pixel.
// - NeoPixel strip's DATA-IN should pass through a 300-500 OHM RESISTOR.
// - AVOID connecting NeoPixels on a LIVE CIRCUIT. If you must, ALWAYS
//   connect GROUND (-) first, then +, then data.
// - When using a 3.3V microcontroller with a 5V-powered NeoPixel strip,
//   a LOGIC-LEVEL CONVERTER on the data line is STRONGLY RECOMMENDED.
// (Skipping these may work OK on your workbench but can fail in the field)

#include <Adafruit_CircuitPlayground.h>

#ifdef __AVR__
 #include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
// On a Trinket or Gemma we suggest changing this to 1:
#define LED_PIN    9

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 120

// Declare our NeoPixel strip object:
Adafruit_CPlay_NeoPixel boardleds(10, 17, NEO_GRB + NEO_KHZ800);
Adafruit_CPlay_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
// Argument 1 = Number of pixels in NeoPixel strip
// Argument 2 = Arduino pin number (most are valid)
// Argument 3 = Pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)


// FidgetSpinner stuff for accelerometer code
// Configure which accelerometer axis to check:
// This should be one of the following values (uncomment only ONE):
//#define AXIS              CircuitPlayground.motionX()    // X axis, good for Circuit Playground express (SAMD21).
//#define AXIS                CircuitPlayground.motionY()    // Y axis, good for Circuit Playground classic (32u4).
#define AXIS              CircuitPlayground.motionZ()    // Z axis, wacky--try it!

// Configure pixel brightness.
// Set this to a value from 0 to 255 (minimum to maximum brightness).
#define BRIGHTNESS          255

// Debounce times for peak detection and button presses.
// You probably don't need to change these.
#define PEAK_DEBOUNCE_MS     500
#define BUTTON_DEBOUNCE_MS   20

// Uncomment this to output a stream of debug information
// from the accelerometer and peak detector.  Use the serial
// plotter to view the graph of these 4 values:
//  - Accelerometer y axis value (in m/s^2)
//  - Peak detector filtered y axis average
//  - Peak detector filtered y axis standard deviation
//  - Peak detector result, 0=no peak 1=positive peak, -1=negative peak
//#define DEBUG


// Global state used by the sketch (you don't need to change this):

uint32_t lastMS = millis();
uint32_t peakDebounce = 0;
uint32_t buttonDebounce = 0;
bool lastButton1;
bool lastButton2;
int currentAnimation = 0;
int currentColor = 0;
int currentOrientation = 0;  // track turning globe/board over and then back
uint32_t rainColor = 0;      // for interruptable rainbows, this needs to be a global
int rainbowIncrement = 70;   // you decide how fast the rainbow changes

// setup() function -- runs once at startup --------------------------------

void setup() {
  // These lines are specifically to support the Adafruit Trinket 5V 16 MHz.
  // Any other board, you can remove this part (but no harm leaving it):
#if defined(__AVR_ATtiny85__) && (F_CPU == 16000000)
  clock_prescale_set(clock_div_1);
#endif
  // END of Trinket-specific code.

    // Initialize serial output for debugging and plotting.
  Serial.begin(115200);
  // Initialize Circuit Playground library and set accelerometer
  // to its widest +/-16G range.
  CircuitPlayground.begin(BRIGHTNESS);
  CircuitPlayground.setAccelRange(LIS3DH_RANGE_16_G);
  // Initialize starting button state.
  lastButton1 = CircuitPlayground.leftButton();
  lastButton2 = CircuitPlayground.rightButton();

  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)
  boardleds.begin();           // INITIALIZE NeoPixel strip (circuit playground board) object (REQUIRED)
  boardleds.show();            // Turn OFF all pixels ASAP
  boardleds.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)

}


// loop() function -- runs repeatedly as long as board is on ---------------

void loop() {
// Update time since last loop call.
  uint32_t currentMS = millis();
  uint32_t deltaMS = currentMS - lastMS;  // Time in milliseconds.
  float deltaS = deltaMS / 1000.0;        // Time in seconds.
  lastMS = currentMS;

  // Grab the current accelerometer axis value and look for a sudden peak.
  float accel = AXIS;

  // If in debug mode, print out the current acceleration and peak detector
  // state (average, standard deviation, and peak result).  Use the serial
  // plotter to view this over time.
  #ifdef DEBUG
  Serial.print(CircuitPlayground.motionZ());
  Serial.print(",");
  Serial.println(currentOrientation);
  #endif

  // If there was a peak and enough time has elapsed since the last peak
  // (i.e. to 'debounce' the noisey peak signal a bit) then start the spinner
  // moving at a velocity proportional to the accelerometer value.
  if (currentMS >= peakDebounce) {
    peakDebounce = currentMS + PEAK_DEBOUNCE_MS;
    
    if (currentOrientation == 0){
      if (CircuitPlayground.motionZ()<0.0) {
        Serial.println("Upside Down");
        currentOrientation = 1;
       }
    }else{
      if (CircuitPlayground.motionZ()>0.0) {
        currentOrientation = 0;
        currentAnimation = (currentAnimation + 1) % 7;
      }
    }
  }

 // Check if enough time has passed to test for button releases.
  if (currentMS >= buttonDebounce) {
    buttonDebounce = currentMS + BUTTON_DEBOUNCE_MS;
    // Check if any button was released.  I.e. the last known button
    // state was pressed (true) and the current state is released (false).
    bool button1 = CircuitPlayground.leftButton();
    bool button2 = CircuitPlayground.rightButton();
    if (!button1 && lastButton1) {
      // Button 1 released!
      // Change to the next animation (looping back to start after 3 since
      // there are 4 total animations).
      currentAnimation = (currentAnimation + 1) % 7;
    }
//    if (!button2 && lastButton2) {
//      // Button 2 released!
//    }
    lastButton1 = button1;
//    lastButton2 = button2;
  }

  // draw the current animation frame.
   uint32_t scolor;  // colour of the tracing set for the strip
   uint32_t tail;    // colour of the tail, after the tracing set for the strip
   uint32_t bcolor;  // "background" brightness/colour for the board
   uint32_t rcolor;  // "foreground" brightness/colour for the board
       
   switch (currentAnimation) {
    case 0:   // white
      scolor = strip.Color(255,255,255);
      tail = strip.Color(10,10,10);
      bcolor = strip.Color(255,255,255);
      rcolor = strip.Color(0,0,0);
      break;
    case 1:   // red
      scolor = strip.Color(255,0,0);
      tail = strip.Color(10,0,0);
      bcolor = strip.Color(255,0,0);
      rcolor = strip.Color(0,0,0);
      break;
    case 2:   // green
      scolor = strip.Color(0,255,0);
      tail = strip.Color(0,10,0);
      bcolor = strip.Color(0,255,0);
      rcolor = strip.Color(0,0,0);
      break;
    case 3:   // blue
      scolor = strip.Color(0,0,255);
      tail = strip.Color(0,0,10);
      bcolor = strip.Color(0, 0, 255);
      rcolor = strip.Color(0,0,0);
      break;
    case 4:   // white with rainbow tail
      scolor = strip.Color(255,255,255);
      tail = strip.ColorHSV(rainColor, 255, 40); // tail not as bright as tracer section
      bcolor = strip.ColorHSV(rainColor);        // board same colour as tail right after tracer
      rcolor = strip.Color(0,0,0);
      break;
    case 5:   //rainbow with rainbow tail
      scolor = strip.ColorHSV(rainColor);
      tail = strip.ColorHSV(rainColor+256, 255, 40); 
      bcolor = strip.ColorHSV(rainColor);
      rcolor = strip.Color(0,0,0);
      break;
    case 6:   // mellow rainbow, no tracing
      bcolor = strip.ColorHSV(rainColor);
      rcolor = strip.Color(0,0,0);
      break;
 
    }

    if (currentAnimation==6){
      rainbowSet(getTime(10, strip.numPixels()));
    } else {
      travellingSet(scolor, tail, 4, getTime(50,strip.numPixels()), 0, 1);
    }
    brightnessSet(bcolor,rcolor,getTime(50,boardleds.numPixels()),0,0);
}


// Some functions of our own for creating animated effects -----------------

// animate from one colour to another, usually brightness by setting the "to" colour to black
void brightnessSet(uint32_t fromcolor, uint32_t tocolor, int offset, int wait, int sorb) {
  int fr = (fromcolor & 0xFF0000) >> 16;
  int fg = (fromcolor & 0x00FF00) >> 8;
  int fb = (fromcolor & 0x0000FF);
  int tr = (tocolor & 0xFF0000) >> 16;
  int tg = (tocolor & 0x00FF00) >> 8;
  int tb = (tocolor & 0x0000FF);
  
  if(sorb > 0) {  // strip or board
    int n = strip.numPixels();
    
    for(int i=0; i<n; i++) { // For each pixel in strip...
      strip.setPixelColor((i+offset) % n, fr+((tr-fr)/n)*i,fg+((tg-fg)/n)*i,fb+((tb-fb)/n)*i);         //  Set pixel's color
      strip.show();                          //  Update strip to match
       }
    }   
  else { 
    int n = boardleds.numPixels();

    for(int i=0; i<n; i++) { // For each pixel in strip...
      boardleds.setPixelColor((i+offset) % n, fr+((tr-fr)/n)*i,fg+((tg-fg)/n)*i,fb+((tb-fb)/n)*i);         //  Set pixel's color
      boardleds.show();                          //  Update strip to match
    }
  }
  delay(wait);                           //  Pause for a moment
}

// animate the travelling tracer and the head of the tail
void travellingSet(uint32_t color, uint32_t tail, int len, int offset, int wait, int sorb) {
  rainColor+=rainbowIncrement;
  if(sorb > 0) {
    int n = strip.numPixels();
    for(int i=0; i<len; i++) { // for the requested length
        strip.setPixelColor((i+offset) % n, color);         //  Set pixel's color (in RAM)
        strip.show();                          //  Update strip to match
    } 
    delay(wait);                           //  Pause for a moment
    strip.setPixelColor(offset, tail);
  } else {
    int n = boardleds.numPixels();
    for(int i=0; i<len; i++) { // for the requested length
        boardleds.setPixelColor((i+offset) % n, color);         //  Set pixel's color (in RAM)
        boardleds.show();                          //  Update strip to match
    } 
    delay(wait);                           //  Pause for a moment
    boardleds.setPixelColor(offset, tail);
  }
}

// Rainbow (see below) modified to be interruptable
void rainbowSet(int wait) {
  // Hue of first pixel runs 5 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 5*65536. Adding 256 to firstPixelHue each time
  // means we'll make 5*65536/256 = 1280 passes through this outer loop:
  int count = strip.numPixels();
  long firstPixelHue = (rainColor+=rainbowIncrement);
    for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
      // Offset pixel hue by an amount to make one full revolution of the
      // color wheel (range of 65536) along the length of the strip
      // (strip.numPixels() steps):
      int pixelHue = firstPixelHue + (i * 65536L / count);
      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the single-argument hue variant. The result
      // is passed through strip.gamma32() to provide 'truer' colors
      // before assigning to each pixel:
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue, 255, 100)));
    }
    strip.show(); // Update strip with new contents
    delay(wait);  // Pause for a moment
}

// --------------------------------------------------------------------------------
// example code modified to combine effects for the LED strip and the board lights
// tacked onto the end.  for testing purposes.
// --------------------------------------------------------------------------------
// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.
void colorWipe(uint32_t color, int wait) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
  for(int i=0; i<boardleds.numPixels(); i++) { // For each pixel in strip...
    boardleds.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    boardleds.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}

// Theater-marquee-style chasing lights. Pass in a color (32-bit value,
// a la strip.Color(r,g,b) as mentioned above), and a delay time (in ms)
// between frames.
void theaterChase(uint32_t color, int wait) {
  for(int a=0; a<10; a++) {  // Repeat 10 times...
    for(int b=0; b<3; b++) { //  'b' counts from 0 to 2...
      strip.clear();         //   Set all pixels in RAM to 0 (off)
      boardleds.clear();
      // 'c' counts up from 'b' to end of strip in steps of 3...
      for(int c=b; c<strip.numPixels(); c += 3) {
        strip.setPixelColor(c, color); // Set pixel 'c' to value 'color'
      }
      for(int c=b; c<boardleds.numPixels(); c += 3) {
        boardleds.setPixelColor(c, color); // Set pixel 'c' to value 'color'
      }
      strip.show(); // Update strip with new contents
      boardleds.show();
      delay(wait);  // Pause for a moment
    }
  }
}

// Rainbow cycle along whole strip. Pass delay time (in ms) between frames.
void rainbow(int wait) {
  // Hue of first pixel runs 5 complete loops through the color wheel.
  // Color wheel has a range of 65536 but it's OK if we roll over, so
  // just count from 0 to 5*65536. Adding 256 to firstPixelHue each time
  // means we'll make 5*65536/256 = 1280 passes through this outer loop:
  int count = strip.numPixels() + boardleds.numPixels();
  for(long firstPixelHue = 0; firstPixelHue < 5*65536; firstPixelHue += 256) {
    for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
      // Offset pixel hue by an amount to make one full revolution of the
      // color wheel (range of 65536) along the length of the strip
      // (strip.numPixels() steps):
      int pixelHue = firstPixelHue + (i * 65536L / count);
      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the single-argument hue variant. The result
      // is passed through strip.gamma32() to provide 'truer' colors
      // before assigning to each pixel:
      strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
    }
    for(int i=0; i<boardleds.numPixels(); i++) { // For each pixel in strip...
      // Offset pixel hue by an amount to make one full revolution of the
      // color wheel (range of 65536) along the length of the strip
      // (strip.numPixels() steps):
      int pixelHue = firstPixelHue + ((i+strip.numPixels()) * 65536L / count);
      // strip.ColorHSV() can take 1 or 3 arguments: a hue (0 to 65535) or
      // optionally add saturation and value (brightness) (each 0 to 255).
      // Here we're using just the single-argument hue variant. The result
      // is passed through strip.gamma32() to provide 'truer' colors
      // before assigning to each pixel:
      boardleds.setPixelColor(i, boardleds.gamma32(strip.ColorHSV(pixelHue)));
    }
    strip.show(); // Update strip with new contents
    boardleds.show();
    delay(wait);  // Pause for a moment
  }
}

// timing stuff from FidgetSpinner code
int getTime(uint32_t stepsize, int pixels) {
  uint32_t currentMS = millis();
  int position = (currentMS/stepsize) % pixels;
  return position;
}
